import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;

public class ClientListenerThread extends Thread {
	private static int nr_clients = 0;
	private int client_number = ++nr_clients;
	private final SimpleLogger logger = new SimpleLogger("ClientListenerThread " + client_number);
	private final Socket socket;
	private final BufferedReader from_client;
	private final PrintWriter to_client;
	private final String bookFileName;
	private String text = "";

	public ClientListenerThread(Socket socket, String bookFileName) throws IOException {
		logger.log("Constructing ClientListenerThread");
		this.socket = socket;
		from_client = new BufferedReader(new InputStreamReader(socket.getInputStream()));
		to_client = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);
		this.bookFileName = bookFileName;
	}

    private boolean quit = false;
    private boolean commit = false;

    public void run() {
		logger.log("Starting ClientListenerThread.run");
		while (! quit) {
            try {
            	String line_from_client = from_client.readLine();
                logger.log("From client: '" + line_from_client + "'");
                if (line_from_client == null) {
                	quit = true;
                }
                else if (line_from_client.equals("COMMIT")) {
                	commit = true;
                    quit = true;
            	}
                else if (line_from_client.equals("ROLLBACK")) {
                    text = "";
            	}
                else {
                	text = text + line_from_client + "\n";
                }
            }
            catch (IOException e) {
                logger.log("Lost contact with client.");
                e.printStackTrace();
                quit = true;
            }
        } // while
		try {
			socket.close();
		}
		catch (IOException e) {
			logger.log("Failed to close the socket! Exception: " + e);
			e.printStackTrace();
		}
        logger.log("Client has finished sending! commit = " + commit + " text = '" + text + "'");
        if (commit)
        	appendToBookFile(text);
    } // run

	private void appendToBookFile(String text) {
		try {
			synchronized ("BookWriterLook") {
				FileWriter fw = new FileWriter(bookFileName, true);
				BufferedWriter bw = new BufferedWriter(fw);
				PrintWriter out = new PrintWriter(bw);
				logger.log("Writing to the book file: '" + text + "'");
				out.println(text);
				out.close();
			}
		}
		catch (IOException e) {
			logger.log("Failed to write the book file! Exception: " + e);
			e.printStackTrace();
		}
	}
} // ClientListenerThread
